"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const debug_1 = __importDefault(require("debug"));
const rascal_1 = require("rascal");
const publication_1 = __importDefault(require("./publication"));
const subscription_1 = __importDefault(require("./subscription"));
const debug = debug_1.default("rascal:ProxyBroker");
class BrokerProxy {
    constructor(config, components) {
        this.publications = new Map();
        this.subScriptions = new Map();
        this.config = config;
        this.components = components;
    }
    static create(config, components) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.instance) {
                this.instance = new BrokerProxy(config, components || {});
                yield this.instance.init();
            }
            return this.instance;
        });
    }
    connect(name) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.broker.connect(name);
        });
    }
    nuke() {
        return __awaiter(this, void 0, void 0, function* () {
            return this.broker.nuke();
        });
    }
    purge() {
        return __awaiter(this, void 0, void 0, function* () {
            return this.broker.purge();
        });
    }
    shutdown() {
        return __awaiter(this, void 0, void 0, function* () {
            return this.broker.shutdown();
        });
    }
    bounce() {
        return __awaiter(this, void 0, void 0, function* () {
            return this.broker.bounce();
        });
    }
    /**
     * publish message
     * @param name publication name
     * @param message message
     * @param overrides overrides
     */
    publish(name, message, overrides) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.getPublication(name).publish(message, overrides);
        });
    }
    /**
     * direct subscribe use rascal
     * @param name subscription name
     * @param overrides overrides options
     */
    subscribe(name, overrides) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.broker.subscribe(name, overrides);
        });
    }
    /**
     * add Consumer instance to receive message
     * @param consumer subclass of Consumer
     */
    addConsumer(consumer) {
        return __awaiter(this, void 0, void 0, function* () {
            const subscription = yield this.getSubScription(consumer);
            return subscription.subscribe();
        });
    }
    getPublication(name) {
        let publication = this.publications.get(name);
        if (!publication) {
            publication = new publication_1.default(name, this.broker);
            this.publications.set(name, publication);
        }
        return publication;
    }
    getSubScription(consumer) {
        return __awaiter(this, void 0, void 0, function* () {
            let subScription = this.subScriptions.get(consumer.name);
            if (!subScription) {
                subScription = new subscription_1.default(consumer, this.broker);
                this.subScriptions.set(consumer.name, subScription);
            }
            return subScription;
        });
    }
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            this.broker = yield rascal_1.BrokerAsPromised.create(rascal_1.withDefaultConfig(this.config), this.components);
            this.attachBrokerHandlers();
        });
    }
    attachBrokerHandlers() {
        this.broker.on("error", this.onBrokerError.bind(this));
        this.broker.on("connect", this.onBrokerConnect.bind(this));
        this.broker.on("disconnect", this.onBrokerDisconnect.bind(this));
    }
    onBrokerError(err) {
        return __awaiter(this, void 0, void 0, function* () {
            debug("Broker Error: %s", err.message);
            this.stopAllTimer();
        });
    }
    onBrokerConnect() {
        return __awaiter(this, void 0, void 0, function* () {
            this.runAllTimer();
        });
    }
    onBrokerDisconnect() {
        return __awaiter(this, void 0, void 0, function* () {
            this.stopAllTimer();
        });
    }
    runAllTimer() {
        for (const publication of this.publications.values()) {
            publication.runTimer();
        }
    }
    stopAllTimer() {
        for (const publication of this.publications.values()) {
            publication.pauseTimer();
        }
    }
}
exports.BrokerProxy = BrokerProxy;
