"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const debug_1 = __importDefault(require("debug"));
const ramda_1 = __importDefault(require("ramda"));
const debug = debug_1.default("rascal:ProxySubscription");
class SubScription {
    constructor(consumer, broker) {
        this.consumer = consumer;
        this.broker = broker;
    }
    subscribe() {
        return __awaiter(this, void 0, void 0, function* () {
            this.session = yield this.broker.subscribe(this.consumer.name, this.consumer.overrides);
            this.attachSessionHandlers();
            return this.session;
        });
    }
    messageHandler(message, content, ackOrNack) {
        return __awaiter(this, void 0, void 0, function* () {
            const once = ramda_1.default.once(ackOrNack);
            try {
                yield this.consumer.onMessage(content, message, once);
                once();
            }
            catch (error) {
                once(error, this.getRecovery());
            }
        });
    }
    onInvalidContent(err, message, ackOrNack) {
        return __awaiter(this, void 0, void 0, function* () {
            const once = ramda_1.default.once(ackOrNack);
            if (this.consumer.onInvalidContent) {
                try {
                    yield this.consumer.onInvalidContent(err, message, once);
                }
                catch (error) {
                    once(error);
                }
            }
            once(err);
        });
    }
    onRedeliveriesError(err, message, ackOrNack) {
        return __awaiter(this, void 0, void 0, function* () {
            const once = ramda_1.default.once(ackOrNack);
            if (this.consumer.onRedeliveriesError) {
                try {
                    yield this.consumer.onRedeliveriesError(err, message, once);
                }
                catch (error) {
                    once(error);
                }
            }
            once(err);
        });
    }
    onRedeliveriesExceeded(err, message, ackOrNack) {
        return __awaiter(this, void 0, void 0, function* () {
            const once = ramda_1.default.once(ackOrNack);
            if (this.consumer.onRedeliveriesExceeded) {
                try {
                    yield this.consumer.onRedeliveriesExceeded(err, message, once);
                }
                catch (error) {
                    once(error);
                }
            }
            once(err);
        });
    }
    onError(err) {
        return __awaiter(this, void 0, void 0, function* () {
            debug("Subscription error %", err.message);
        });
    }
    onConsumerCancel(err) {
        return __awaiter(this, void 0, void 0, function* () {
            debug("Subscription consumer cancel %", err.message);
        });
    }
    getRecovery() {
        var _a;
        const consumeRecovery = ((_a = this.broker.config.subscriptions) === null || _a === void 0 ? void 0 : _a[this.consumer.name].recovery) || "";
        const allRecovery = this.broker.config.recovery || {};
        return allRecovery[consumeRecovery] || [];
    }
    attachSessionHandlers() {
        this.session.on("message", this.messageHandler.bind(this));
        this.session.on("error", this.onError.bind(this));
        this.session.on("invalid_content", this.onInvalidContent.bind(this));
        this.session.on("redeliveries_error", this.onRedeliveriesError.bind(this));
        this.session.on("redeliveries_exceeded", this.onRedeliveriesExceeded.bind(this));
        this.session.on("cancelled", this.onConsumerCancel.bind(this));
    }
}
exports.default = SubScription;
